import React, { useCallback } from 'react';
import { CustomTable } from "../common/table/CustomTable";
import { Grid, Tooltip } from "@material-ui/core";
import { formatDate, noWrapText } from '../common/Helper';
import { useRedirect } from '../../hooks/Redirect';
import { JobStatusIcon, JobTypeIcon } from './JobIcons';

const formatJobDate = (value) => {
  if (value) {
    return formatDate(value);
  }
  return "Not started yet";
}

const jobLogColumns = [
  { id: 'type', label: 'Type', width: '8ch', textAlign: "center"},
  { id: 'ioc', label: 'IOC Name', width: '15ch', sortable: true },
  { id: 'host', label: 'Host', width: '15ch', sortable: false },
  { id: 'network', label: 'Network', width: '10ch', sortable: false },
  { id: 'user', label: 'User', width: '10ch', sortable: true },
  { id: 'version', label: 'Reference', width: '10ch', sortable: true },
  { id: 'start', label: 'Time', width: '15ch', sortable: true }
];

const iocServiceControlColumns = [
  { id: 'status', label: 'Status', width: '4ch', textAlign: "center"},
  { id: 'type', label: 'Command', width: '4ch', textAlign: "center"},
  { id: 'user', label: 'User', width: '10ch' },
  { id: 'start', label: 'Time', width: '15ch' }
];

function createGitVersionField(version, shortVersion) {
  return (
    <Tooltip title={version} arrow enterDelay={400}>
      <label>{noWrapText(shortVersion ?? version)}</label>
    </Tooltip>
  )
}

function createTableRow(job, colorStyle) {
  return {
    id: job.id,
    type:
      <Grid container direction="column" justifyContent="center" alignItems="center">
        <JobTypeIcon type={job.type} colorStyle={colorStyle}/>
      </Grid>,
    status:
      <Grid container direction="column" justifyContent="center" alignItems="center">
       <JobStatusIcon status={job.status} />
      </Grid>,
    ioc: noWrapText(job.iocName),
    version: createGitVersionField(job.gitReference, job.sourceVersionShort),
    host: noWrapText(job.host?.hostName ?? null),
    network: noWrapText(job.host?.network ?? null),
    user: noWrapText(job.createdBy),
    start: noWrapText(formatJobDate(job.time)),
    deploymentFailed: job.status === 'FAILED' || job.status === 'ERROR',
    jobType: job.type,
    iocId : job.iocId
  };
}

function createTableRowJobLog(job) {
  return createTableRow(job, "black")
}

export function JobTable({ jobs, rowType = "jobLog", totalCount, lazyParams, setLazyParams, columnSort, setColumnSort, rowsPerPage, loading }) {
  const tableTypeSpecifics = {
    "jobLog": [jobLogColumns, createTableRowJobLog],
    "iocServiceControl": [iocServiceControlColumns, createTableRow]
  }
  
  const redirect = useRedirect();
  const [columns, createRow] = tableTypeSpecifics[rowType];

  const itemLink = useCallback((id, item) => {
    if (["DEPLOY","UNDEPLOY"].includes(item.jobType)) {
      return `/deployments/${id}`
    }
    else {
      return `/iocs/${item.iocId}` // TODO: This currently doesn't work. Waiting for update to endpoint to include iocId
    }
  }, []); 
  const onRowClicked = useCallback((id, item) => {
    redirect(itemLink(id, item));
  }, [itemLink, redirect]);

  return (
    <CustomTable columns={columns} rows={jobs.map(job => createRow(job))} handleRowClick={onRowClicked} itemLink={itemLink}
      totalCount={totalCount} lazyParams={lazyParams} setLazyParams={setLazyParams} columnSort={columnSort} setColumnSort={setColumnSort}
      rowsPerPage={rowsPerPage} loading={loading} />
  );
}